@@ -3,9 +3,11 @@ package Data::Mining::Apriori;
 use 5.010001;
 use strict;
 use warnings;
+use threads;
+use threads::shared;
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = 0.03;
+our $VERSION = 0.04;
 
 my $self;
 
@@ -13,6 +15,19 @@ $" = ', ';
 $| = 1;
 $SIG{'INT'} = \&stop;
 
+my $spin : shared = 1;
+
+async{
+	while($spin){
+		do{
+			select undef, undef, undef, 0.1;
+			printf "$_\b";
+		}
+		for qw(/ - \ |);
+	}
+}
+->detach;
+
 sub new{
 	my $type = shift;
 	my $class = ref($type)||$type;
@@ -21,7 +36,7 @@ sub new{
 		minSupport => undef,
 		minConfidence => undef,
 		output => undef,
-		messages => undef,
+		messages => shift,
 		itemsKeyDescription => undef,
 		itemsKeyTransactions => undef,
 		quantityPossibleRules => undef,
@@ -29,11 +44,10 @@ sub new{
 		recursively => 0,
 		associationRules => undef,
 		itemsetSize => undef,
-		rule => 0,
-		excel => undef,
-		workbook => undef
+		rule => 0
 	};
 	@{$self->{frequentItemset}}=();
+	$spin=$self->{messages};
 	bless($self,$class);
 	return $self;
 }
@@ -59,7 +73,7 @@ sub generate_rules{
 	if($self->{messages}){
 		my $quantityItems = scalar(@itemsetCandidateSizeOne);
 		$self->{quantityPossibleRules} = ((3**$quantityItems)-(2**($quantityItems+1))+1);
-		print "\n$quantityItems items, $self->{quantityPossibleRules} possible rules ...";
+		print "\n$quantityItems items, $self->{quantityPossibleRules} possible rules";
 	}
 	$self->{recursively} = 1;
 	$self->association_rules_itemset_size(\@itemsetCandidateSizeOne, 2);
@@ -85,8 +99,8 @@ sub association_rules_itemset_size{
 	}
 	my @newItemsetCandidateSizeOne = ();
 	if($self->{messages}){
-		print "\n\nItemset size ", $self->{itemsetSize}, ", ", scalar(@itemsetCandidateSizeOne), " items ...";
-		print "\nProcessing ...";
+		print "\n\nItemset size ", $self->{itemsetSize}, ", ", scalar(@itemsetCandidateSizeOne), " items ";
+		print "\nProcessing... ";
 	}
 	for(my$itemsetSize=2;$itemsetSize<=$itemsetSizeLimit;$itemsetSize++){
 		my $code=<<'CODE';
@@ -112,11 +126,7 @@ for(my$subsetSize=0;$subsetSize<$#itemsetCandidate;$subsetSize++){
 		push @subsetLeft,$itemsetCandidate[$m];
 	}
 	@subsetLeft = sort @subsetLeft;
-	foreach my$item(@itemsetCandidate){
-		if(!($item ~~ @subsetLeft)){
-			push @subsetRight,$item;
-		}
-	}
+	@subsetRight = grep!($_~~@subsetLeft),@itemsetCandidate;
 	@subsetRight = sort @subsetRight;
 	if("@subsetRight" ~~ @{$itemsetSubset{"@subsetLeft"}}){
 		next;
@@ -141,19 +151,17 @@ for(my$subsetSize=0;$subsetSize<$#itemsetCandidate;$subsetSize++){
 	if($support >= $self->{minSupport}){
 		my $confidence = sprintf("%.2f",(($supportAssociationRight/$supportAssociationLeft)*100));
 		if($confidence >= $self->{minConfidence}){
-			$support =~ s/\./,/;
-			$confidence =~ s/\./,/;
 			$self->{rule}++;
 			@{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule}} = ("{ @subsetLeft } => { @subsetRight }", $support, $confidence);
 			@{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}} = ();
+			push @{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}},@subsetLeft;
 			foreach my$item(@subsetLeft){
-				push @{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}},$item;
 				if(!($item ~~ @newItemsetCandidateSizeOne)){
 					push @newItemsetCandidateSizeOne,$item;
 				}
 			}
+			push @{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}},@subsetRight;			
 			foreach my$item(@subsetRight){
-				push @{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}},$item;
 				if(!($item ~~ @newItemsetCandidateSizeOne)){
 					push @newItemsetCandidateSizeOne,$item;
 				}
@@ -176,7 +184,7 @@ CODE
 			eval $code;
 			die("Error: $@\n") if $@;
 			if($self->{messages}){
-				print "\nFrequent itemset: { @newItemsetCandidateSizeOne }, ", scalar(@newItemsetCandidateSizeOne), " items ...";
+				print "\nFrequent itemset: { @newItemsetCandidateSizeOne }, ", scalar(@newItemsetCandidateSizeOne), " items ";
 			}
 			if($self->{associationRules}){
 				@{$self->{frequentItemset}}=@newItemsetCandidateSizeOne;
@@ -192,16 +200,22 @@ CODE
 }
 
 sub stop{
-	print "\nStopping ...";
-	$self->output if $self->{associationRules};
-	print "\nExit? (Y/N): ";
-	my $exit = <STDIN>;
-	chomp($exit);
-	if($exit =~ /^y$/i){
-		exit;
+	if($self->{messages}){
+		print "\nStopping... ";
+		$self->output if $self->{associationRules};
+		print "\nExit? (Y/N): ";
+		my $exit = <STDIN>;
+		chomp($exit);
+		if($exit =~ /^y$/i){
+			exit;
+		}
+		else{
+			print "\nProcessing... ";
+		}
 	}
 	else{
-		print "\nProcessing ...";
+		$self->output if $self->{associationRules};
+		exit;
 	}
 }
 
@@ -218,18 +232,20 @@ sub output{
 
 sub file{
 	if($self->{messages}){
-		print "\nExporting to file \"output_itemset_size_$self->{itemsetSize}.txt\" ...";
+		print "\nExporting to file \"output_itemset_size_$self->{itemsetSize}.txt\"... ";
 	}
 	open(FILE,">output_itemset_size_$self->{itemsetSize}.txt");
-	print FILE "Rule\tSupport %\tConfidence %\n";
+	print FILE "Rules\tSupport %\tConfidence %\n";
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
-		print FILE "R$rule\t@{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1]\t@{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2]\n";
+		${$self->{associationRules}{$rule}{"R$rule"}{rule}}[1]=~s/\./,/;
+		${$self->{associationRules}{$rule}{"R$rule"}{rule}}[2]=~s/\./,/;
+		print FILE "R$rule\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1]\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2]\n";
 	}
 	print FILE "\n";
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
-		print FILE "Rule R$rule: @{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[0]\n";
-		print FILE "Support: @{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1] %\n";
-		print FILE "Confidence: @{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %\n";
+		print FILE "Rule R$rule: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[0]\n";
+		print FILE "Support: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1] %\n";
+		print FILE "Confidence: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %\n";
 		print FILE "Items:\n";
 		foreach my$item(@{$self->{associationRules}{$rule}{"R$rule"}{items}}){
 			print FILE "$item $self->{itemsKeyDescription}{$item}\n";
@@ -245,66 +261,68 @@ sub file{
 }
 
 sub excel{
-	require Win32::OLE;
-	my($sheet,$range,$chart);
+	require Excel::Writer::XLSX;
 	if($self->{messages}){
-		print "\nExporting to excel ...";
+		print "\nExporting to excel \"output_itemset_size_$self->{itemsetSize}.xlsx\"... ";
 	}
-	if(!$self->{excel}){
-		$self->{excel} = Win32::OLE->new('Excel.Application');
-		$self->{workbook} = $self->{excel}->Workbooks->Add;
-	}
-	$self->{workbook}->Worksheets->Add;
-	$sheet = $self->{workbook}->Worksheets(1);
-	$sheet->{Name} = "Itemset Size $self->{itemsetSize}";
-	$sheet->Range("A1:C1")->{Value} = ["Rule","Support %","Confidence %"];
-	my $i = 1;
+	my $workbook  = Excel::Writer::XLSX->new("output_itemset_size_$self->{itemsetSize}.xlsx");
+	my $worksheet = $workbook->add_worksheet();
+	my $bold = $workbook->add_format(bold => 1);
+	my $headings = ['Rules', 'Support %', 'Confidence %'];
+	my(@rules,@support,@confidence);	
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
-		$i++;
-		$sheet->Range("A$i:C$i")->{Value} = ["R$rule",@{$self->{associationRules}{$rule}{"R$rule"}{rule}}[1],@{$self->{associationRules}{$rule}{"R$rule"}{rule}}[2]];
+		push @rules,"R$rule";
+		push @support,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[1];
+		push @confidence,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[2];
 	}
-	my $j = $i + 1;
+	my$end=(scalar(@rules)+1);
+	my@data=(\@rules,\@support,\@confidence);
+	$worksheet->write('A1', $headings, $bold);
+	$worksheet->write('A2', \@data);
+	my$chart=$workbook->add_chart(type => 'column', embedded => 1);
+	$chart->add_series(
+		name       => 'Support %',
+		categories => '=Sheet1!$A$2:$A$'.$end,
+		values     => '=Sheet1!$B$2:$B$'.$end,
+	);
+	$chart->add_series(
+		name       => 'Confidence %',
+		categories => '=Sheet1!$A$2:$A$'.$end,
+		values     => '=Sheet1!$C$2:$C$'.$end,
+	);
+	$worksheet->insert_chart('H2', $chart);
+	my$line=($end+2);
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
-		$j++;
-		$sheet->Range("A$j")->{Value} = "Rule R$rule: @{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[0]";
-		$j++;
-		$sheet->Range("A$j")->{Value} =  "Support: @{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1] %";
-		$j++;
-		$sheet->Range("A$j")->{Value} = "Confidence: @{$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %";
-		$j++;
-		$sheet->Range("A$j")->{Value} = "Items:";
+		$worksheet->write("A$line","Rule R$rule: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[0]");
+		$line++;
+		$worksheet->write("A$line","Support: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1] %");
+		$line++;
+		$worksheet->write("A$line","Confidence: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %");
+		$line++;
+		$worksheet->write("A$line","Items:");
+		$line++;
 		foreach my$item(@{$self->{associationRules}{$rule}{"R$rule"}{items}}){
-			$j++;
-			$sheet->Range("A$j")->{Value} = "$item $self->{itemsKeyDescription}{$item}";
+			$worksheet->write("A$line","$item $self->{itemsKeyDescription}{$item}");
+			$line++;
 		}
-		$j++;
+		$line++;
 	}
-	$j++;
-	$sheet->Range("A$j")->{Value} = "Frequent itemset: { @{$self->{frequentItemset}} }";
-	$j++;
-	$sheet->Range("A$j")->{Value} = "Items:";
+	$worksheet->write("A$line","Frequent itemset: { @{$self->{frequentItemset}} }");
+	$line++;
+	$worksheet->write("A$line","Items:");
+	$line++;
 	foreach my$item(@{$self->{frequentItemset}}){
-		$j++;
-		$sheet->Range("A$j")->{Value} = "$item $self->{itemsKeyDescription}{$item}";
+		$worksheet->write("A$line","$item $self->{itemsKeyDescription}{$item}");
+		$line++;
 	}
-	$sheet->ListObjects->Add(1,$sheet->Range("A1:C$i"),1);
-	$chart = $self->{excel}->Charts->Add;
-	$chart->{ChartType} = 51;
-	$chart->SetSourceData($sheet->Range("A1:C$i"),2);
-	$chart->Location(2,$sheet->{Name});
-	$sheet->ChartObjects(1)->{Top} = 10;
-	$sheet->ChartObjects(1)->{Left} = 350;
-	$self->{workbook}->{Saved} = 1;
-	$self->{excel}->{Visible} = 1;
-	$chart = 0;
-	$sheet = 0;
+	$workbook->close();
 }
 
 return 1;
 __END__
 =head1 NAME
 
-Data::Mining::Apriori - Perl extension for implement the data mining algorithm apriori.
+Data::Mining::Apriori - Perl extension for implement the apriori algorithm of data mining.
 
 =head1 SYNOPSIS
 
@@ -324,7 +342,7 @@ Data::Mining::Apriori - Perl extension for implement the data mining algorithm a
 	#        1109          1        1         1
 	#        1110          1        1         1
 
-	my $apriori = new Data::Mining::Apriori; 
+	my $apriori = Data::Mining::Apriori->new(1); # A value boolean to display the messages(optional)
 	 
 	$apriori->{totalTransactions}=10; # The total number of transactions
 
@@ -332,9 +350,7 @@ Data::Mining::Apriori - Perl extension for implement the data mining algorithm a
 
 	$apriori->{minConfidence}=1.55; # The minimum confidence
 
-	$apriori->{output}=1; # The output type {1:export to file, 2:export to excel}(optional)
-
-	$apriori->{messages}=1; # One boolean {1:true, 0:false} to display the messages(optional)
+	$apriori->{output}=1; # The output type (1 - Export to file; 2 - Export to excel)(optional)
 
 	$apriori->{itemsKeyDescription}{'101'}='MILK'; # Hash table to add items by key and description
 	$apriori->{itemsKeyDescription}{102}='BREAD';
@@ -347,7 +363,7 @@ Data::Mining::Apriori - Perl extension for implement the data mining algorithm a
 
 	$apriori->generate_rules; # Generate association rules to no longer meet the minimum support and confidence
 	# or
-	# $apriori->association_rules_itemset_size(3); # Generate rules from a set of items size
+	# $apriori->association_rules_itemset_size(3); # Generate rules from a set of items size 3, for example
 
 	print "\n@{$apriori->{frequentItemset}}\n"; # Show frequent items
 
@@ -418,7 +434,7 @@ Data::Mining::Apriori - Perl extension for implement the data mining algorithm a
 	$query->execute;
 	my $totalTransactions = $query->fetchrow;
 
-	$apriori = new Data::Mining::Apriori;
+	$apriori = Data::Mining::Apriori->new(1);
 
 	$apriori->{totalTransactions}=$totalTransactions;
 
@@ -428,8 +444,6 @@ Data::Mining::Apriori - Perl extension for implement the data mining algorithm a
 
 	$apriori->{output}=1;
 
-	$apriori->{messages}=1;
-
 	my $support = int((($apriori->{totalTransactions}/100)*$apriori->{minSupport}));
 
 	$sql = qq~
@@ -472,7 +486,7 @@ Data::Mining::Apriori - Perl extension for implement the data mining algorithm a
 
 =head1 DESCRIPTION
 
-This module implements apriori data mining algorithm.
+This module implements the apriori algorithm of data mining.
 
 =head1 ATTRIBUTES
 
@@ -490,11 +504,19 @@ The minimum confidence.
 
 =head2 output
 
-The output type {1:export to file, 2:export to excel}(optional).
+The output type (optional):
 
-=head2 messages
+=over 4
 
-A value boolean {1:true, 0:false} to display the messages(optional).
+=item *
+
+1 - Export to file;
+
+=over 4
+
+=item *
+
+2 - Export to excel.
 
 =head2 itemsKeyDescription
 
@@ -514,7 +536,7 @@ Frequent itemset.
 
 =head2 recursively
 
-A value boolean {1:true, 0:false} to generate association rules until no set of items meets the minimum support or minimum confidence.
+A value boolean to generate association rules until no set of items meets the minimum support or minimum confidence.
 
 =head1 METHODS
 
@@ -522,6 +544,12 @@ A value boolean {1:true, 0:false} to generate association rules until no set of
 
 Creates a new instance of Data::Mining::Apriori.
 
+=over 4
+
+=item *
+
+Expected a value boolean to display the messages(optional).
+
 =head2 generate_rules
 
 Generate association rules until no set of items meets the minimum support or minimum confidence.
@@ -532,7 +560,7 @@ Generates association rules by size set of items. Accepts the following argument
 
 =over 4
 
-=item * an integer
+=item * An integer
 
 Representing the size of the set of items.
 
